
(********************************************************************)
(*                                                                  *)
(*   Public-domain Turbo Pascal port of Rich Geldreich's            *)
(*   "PostIt! ver 2.7" : A program to encode a binary file into a   *)
(*                       BASIC source-code file. Type "PIP" at the  *)
(*                       DOS command-line for program syntax.       *)
(*                                                                  *)
(********************************************************************)

 {$A+,B-,D-,E-,F-,I+,N-,O-,R-,S-,V+}
 {$M 1024, 0, 0}

program PrintItPas;
uses
  Dos;

const
              (* Carridge-return constant.                           *)
  coCr            = #13;

              (* Line-feed constant.                                 *)
  coLf            = #10;

              (* Carridge-return + Line-feed constant.               *)
  coCrLf          = coCr + coLf;

              (* Encoded output line-width.                          *)
  coLineWidth     = 68;

              (* Binary Input-buffer size in bytes.                  *)
  coBuffSize      = 10240;

              (* Maximum binary filesize.                            *)
  coMaxBinSize    = 24550;

              (* Encoded output line-count.                          *)
  byLineCount     : byte = 0;

              (* Encoded output file-count.                          *)
  byFileNumber    : byte = 1;

              (* Current bit encoding marker.                        *)
  byCurrentBit    : byte = 0;

              (* Encoded output page-length.                         *)
  byPageLength    : byte = 85;

              (* Encoded output lines to reserve on first page.      *)
  byLines2Reserve : byte = 5;

              (* Number of encoded output characters.                *)
  woCharsOUT      : word = 0;

              (* Encoded output checksum variable.                   *)
  woCheckSum      : word = 0;

              (* Encoded output "extraneous" character-count.        *)
  woExtraIndex    : word = 0;

              (* Binary input buffer-index.                          *)
  woINbuffIndex   : word = 0;

              (* Encoded output buffer-index.                        *)
  woOUTbuffIndex  : word = 0;

              (* Encoded output file-extension.                      *)
  stExtOUT        : extstr = '.BAS';

type
              (* User defined data types.                            *)
  char2           = array[1..2] of char;

  st2             = string[2];
  st20            = string[20];
  st220           = string[220];
  char64          = array[0..63] of char;

              (* User defined buffer types.                          *)
  INbuffType      = array[1..succ(coBuffSize)] of byte;
  OUTbuffType     = array[1..14592] of char;

const
              (* Set of PostIt encoding characters.                  *)
  PostItSet : char64 =
   'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789()';

var
              (* Number of parameters on the command-line.           *)
  byParamNum,

              (* Length of the binary-input string.                  *)
  byOUTnameIndex  : byte;

              (* Binary input file size in bytes.                    *)
  woBinSize,

              (* Variable to check the amount of data read from disk.*)
  woBytesRead,

              (* Variable to check the amount of data written to disk*)
  woBytesWritten : word;

              (* Temporary string variable.                          *)
  stTemp          : st220;

              (* Binary input-file extension string.                 *)
  stExtIN         : extstr;

              (* Binary input-file name string.                      *)
  stNameIN,

              (* Encoded output-file name string.                    *)
  stNameOUT       : namestr;

              (* Binary input-file's directory string.               *)
  stDirIN         : dirstr;

              (* Binary input-file's full path string.               *)
  stPathIN        : pathstr;

              (* Binary input, Encoded output file variables.        *)
  fiIN,
  fiOUT           : file;

              (* Binary input buffer.                                *)
  INbuffer        : INbuffType;

              (* Encoded output buffer.                              *)
  OUTbuffer       : OUTbuffType;


  (***** Routine to handle program errors. Halt if boolean is true.  *)
  procedure ErrorHandler(byErrorNumber : byte; boHalt : boolean);
  begin
              (* Take appropriate action for error-number.           *)
    case byErrorNumber of
       0 : begin
             writeln('  (Syntax)');
             writeln('     PIP <path/filename> (Options) <PL> <LR> ' +
                     '<FP> <FE>', coCrLf);
             writeln('  (Optional Parameters)');
             writeln('     <PL> - Page-length for encoded file. ' +
                     'Min=10 Max=200 (Default 85)');
             writeln('     <LR> - Lines to reserve for the first ' +
                     'encoded page  (Default  5)');
             writeln('     <FP> - File-prefix    1-7 chars');
             writeln('     <FE> - File-extension 1-3 chars' + coCrLf);
             writeln('  (Sample Usage)', coCrLf, coCrLf,
                     '     PIP MYFILE.ZIP');
             writeln('     (Encode MYFILE.ZIP)', coCrLf);
             writeln('     PIP MYFILE.ZIP 95 25 MFPOST ENC');
             writeln('     (Encode MYFILE.ZIP with a page-length of ' +
                     '95 lines, reserve the)');
             writeln('     (first 25 lines of the first page for ' +
                     'notes, use MFPOST as the)');
             writeln('     (encoded file''s name, use ENC as the ' +
                     'encoded file''s extension.)')
           end;
       1 : writeln('File not found ---> ', stPathIN);
       2 : writeln('File exists ---> ', (stNameOUT + stExtOUT));
       3 : writeln('N');
       4 : writeln('Error opening ---> ', stPathIN);
       5 : writeln('Binary file exceeds maximum size of 24,550 bytes');
       6 : writeln('Error creating ---> ', (stNameOUT + stExtOUT));
       7 : writeln('Error writing to ---> ', (stNameOUT + stExtOUT))
    end;
              (* if boHalt boolean is equal to true, then halt       *)
              (* program.                                            *)
    if boHalt then
      halt(byErrorNumber)
  end;        (* ErrorHandler                                        *)


  (***** Convert a string to an integer-type.                        *)
  function Str2Int(stNum : st20) : integer;
  var
    inTemp,
    inError : integer;
  begin
    val(stNum, inTemp, inError);
    if (inError <> 0) then
      Str2Int := -32768
    else
      Str2Int := InTemp
  end;        (* Str2Int                                             *)


  (***** Check if a file exists.                                     *)
  function FileExist(FileName : pathstr) : boolean;
  begin
    FileExist := (FSearch(FileName, '') <> '')
  end;        (* FileExist                                           *)


  (***** Clear the keyboard-buffer.                                  *)
  procedure ClearKeyBuff; assembler;
  asm
    @1: mov ah, 01h
        int 16h
        jz  @2
        mov ah, 00h
        int 16h
        jmp @1
    @2:
  end;        (* ClearKeyBuff                                        *)


  (***** Read a key-press.                                           *)
  function ReadKeyChar : char; assembler;
  asm
    mov ah, 00h
    int 16h
  end;        (* ReadKeyChar                                         *)


  (***** Obtain yes or no response from user.                        *)
  function YesNo : boolean;
  var
    chTemp : char;
  begin
    ClearKeyBuff;
    repeat
      chTemp := upcase(ReadKeyChar)
    until (chTemp in ['Y', 'N']);
    if (chTemp = 'Y') then
      YesNo := true
    else
      YesNo := false
  end;        (* YesNo                                               *)


  (***** Initialize program variables.                               *)
  procedure Init;
  begin
              (* Clear variables.                                    *)
    fillchar(INbuffer, sizeof(INbuffer), 0);
    fillchar(OUTbuffer, sizeof(OUTbuffer), 0);
    fillchar(stPathIN, sizeof(stPathIN), 0);

              (* Record number of command-line parameters.           *)
    byParamNum := paramcount;

              (* If too few or too many parameters used, then        *)
              (* display the program's syntax.                       *)
    if (byParamNum < 1)
    or (byParamNum > 5) then
      ErrorHandler(0, true)
    else
      begin
              (* Assign parmater 1 to Binary input file's path       *)
              (* string.                                             *)
        stPathIN := paramstr(1);

              (* Check if this binary input-file exists.             *)
        if NOT FileExist(stPathIN) then
          ErrorHandler(1, true)
      end;

              (* Expand binary input-file's path to full path string.*)
    stPathIN := fexpand(stPathIN);

              (* Split the full path-string into it's seperate       *)
              (* components.                                         *)
    fsplit(stPathIN, stDirIN, stNameIN, stExtIN);

              (* Assign binary input file name to encoded ouput file *)
              (* name string.                                        *)
    stNameOUT := stNameIN;

              (* If number of command-line parameters is greater     *)
              (* than 1, then...                                     *)
    if (byParamNum > 1) then
      begin
              (* Assign parameter 2's value to page-length variable. *)
        byPageLength := Str2Int(paramstr(2));

              (* If page-length is too small, or too large then      *)
              (* assign the default page-length size.                *)
        if (byPageLength < 10)
        or (byPageLength > 200) then
          byPageLength := 85;

              (* If number of command-line parameters is greater     *)
              (* than 2, then...                                     *)
        if (byParamNum > 2) then
          begin
              (* Assign parameter 3's value to lines-to-reserve      *)
              (* variable.                                           *)
            byLines2Reserve := Str2Int(paramstr(3));

              (* If lines-to-reserve value is too small then assign  *)
              (* the default value.                                  *)
            if (byLines2Reserve < 0) then
              byLines2Reserve := 5
            else
              (* If lines-to-reserve value is too large for page-    *)
              (* length value, then assign it the value of the       *)
              (* page-length minus 10.                               *)
              if (byLines2Reserve > (byPageLength - 10)) then
                byLines2Reserve := (byPageLength - 10);

              (* Check if byLines2Reserve CrLf's will fit in stTemp. *)
              if (byLines2Reserve > 110) then
                byLines2Reserve := 110;

              (* If number of command-line parameters is greater     *)
              (* than 3, then...                                     *)
            if (byParamNum > 3) then
              begin
              (* Assign parameter 4's encoded output-file name to    *)
              (* the encoded output file name variable.              *)
                stNameOUT := paramstr(4);

              (* If number of command-line parameters is greater     *)
              (* than 4, then...                                     *)
                if (byParamNum > 4) then
                  begin
              (* Assign parameter 5's encoded output-file extension  *)
              (* to the encoded output file extension variable.      *)
                    stExtOUT := paramstr(5);

              (* Make sure the length of the encoded output file     *)
              (* extension string is not too long.                   *)
                    if (length(stExtOUT) > 3) then
                      stExtOUT[0] := #3;

              (* Make sure that the the '.' character is the first   *)
              (* character of the encoded output file's extension.   *)
                    if (stExtOUT[1] <> '.') then
                      stExtOUT := '.' + stExtOUT;
                  end
              end
          end
      end;

              (* Make sure the encoded output file's name string is  *)
              (* a maximum of 7 characters.                          *)
    if (length(stNameOUT) > 7) then
      stNameOUT[0] := #7;

              (* Record the length of the encoded output file name   *)
              (* string.                                             *)
    byOUTnameIndex := length(stNameOUT);

              (* Add '1' char to end of output file name string.     *)
    stNameOUT := stNameOUT + '1';

              (* Check to see if the encoded output file name        *)
              (* already exists.                                     *)
    if FileExist(stNameOUT + stExtOUT) then
      begin
              (* Let the user know that the encoded output file name *)
              (* already exists.                                     *)
        ErrorHandler(2, false);

              (* Prompt the user if they want to overwrite this file.*)
        write(coCrLf, 'Overwrite this file? (Y/N) [ ]' + #8#8);

              (* Get user's yes or no response.                      *)
        if YesNo then

              (* User's response was yes.                            *)
          writeln('Y' + coCrLf)

              (* User's response was no. Quit program.               *)
        else
          ErrorHandler(3, true)
      end

  end;        (* init                                                *)


  (***** Convert an integer-type to a string-type.                   *)
  function Int2Str(Number : longint; Width : byte) : st20;
  var
    stTemp : st20;
  begin
    Str(Number:Width, stTemp);
    Int2Str := stTemp
  end;        (* Int2Str                                             *)


  (***** Display the program's title.                                *)
  procedure Title;
  begin
    writeln;
    writeln('PostIt! version 2.7 by Rich Geldreich 1992');
    writeln('Turbo Pascal ver 1.02 by Guy McLoughlin');
    writeln
  end;        (* Title                                               *)


  (***** Open the binary input file for reading.                     *)
  procedure OpenINfile;
  begin
              (* Set the filemode to "read-only" mode.               *)
    filemode := 0;

              (* Attempt to open the binary input file for reading.  *)
    assign(fiIN, stPathIN);
    {$I-}
    reset(fiIN, 1);
    {$I+}
              (* Check if we were successful.                        *)
    if (ioresult <> 0) then
      ErrorHandler(4, true);

              (* Set the filemode to "read/write" mode.              *)
    filemode := 2;

              (* Assign the size of the binary input file in bytes   *)
              (* to the binary-size variable.                        *)
    woBinSize := filesize(fiIN);

              (* Check to see if the binary input file is too big.   *)
    if (woBinSize > coMaxBinSize) then
      ErrorHandler(5, true)
  end;        (* OpenINfile                                          *)


  (***** Open the encoded output file.                               *)
  procedure CreateNewOUTfile(byFileNumber : byte);
  var
    stTemp1 : st2;
  begin
              (* If this is NOT the first encoded output file, then  *)
    if (byFileNumber > 1) then
      begin
              (* Close the last encoded output file.                 *)
        close(fiOUT);

              (* Reset the encoding control variables.               *)
        woINbuffIndex := 0;
        woOUTbuffIndex := 0;
        woExtraIndex  := 0;
        byLineCount := 0;

              (* If this is the 10th encoded output file, then       *)
              (* reduce the length of the output file name by 1 char.*)
        if (byFileNumber = 10)
        and (byOUTnameIndex = 7) then
          dec(byOUTnameIndex);

              (* Create next encoded output file name sting.         *)
        stTemp1 := Int2Str(byFileNumber, 1);
        stNameOUT := copy(stNameOUT, 1, byOUTnameIndex) + stTemp1
      end;

              (* Attempt to open the encoded output file for writing.*)
    assign(fiOUT, (stNameOUT + stExtOUT));
    {$I-}
    rewrite(fiOUT, 1);
    {$I+}
              (* Check to see if we were successful.                 *)
    if (ioresult <> 0) then
      ErrorHandler(6, true);

              (* Let the user know which encoded output file we are  *)
              (* creating.                                           *)
    writeln('Writing : ', (stNameOUT + stExtOUT));

              (* If this is NOT the first encoded output file, then  *)
    if (byFileNumber > 1) then
      begin
              (* Write the first line of the output file.            *)
        stTemp := '''Start of Page ' + Int2Str(byFileNumber, 1) +
                   coCrLf;
        blockwrite(fiOUT, stTemp[1], length(stTemp));

              (* increment the encoded output file line-counter.     *)
        inc(byLineCount)
      end
  end;        (* CreateOUTfile                                       *)


  (***** Create the first encoded output file's "header".            *)
  procedure MainHeader;
  var
    byIndex : byte;
  begin
              (* Initialize the encoded output buffer index variable.*)
    woOUTbuffIndex := 0;

              (* Reserve the lines-to-reserve number of lines in     *)
              (* first encoded output file.                          *)
    stTemp := '';
    for byIndex := 1 to byLInes2Reserve do
      stTemp := stTemp + coCrLf;
    blockwrite(fiOUT, stTemp[1], length(stTemp), woBytesWritten);

              (* Check if ALL data was written to disk.              *)
    if (woBytesWritten <> length(stTemp)) then
      ErrorHandler(7, true);

              (* Advance the encoded output file line-counter by     *)
              (* lines-to-reserve amount.                            *)
    inc(byLineCount, byLines2Reserve);

              (* Write file header to encoded output file.           *)
    stTemp := 'DEFINT A-Z:FOR A=0 TO 6:P(A)=2^A:NEXT:OPEN "B",1,"' +
              stNameIN + stExtIN + '"'+ coCrLf +
              'T$="abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTU' +
              'VWXYZ0123456789()"' + coCrLf;
    blockwrite(fiOUT, stTemp[1], length(stTemp), woBytesWritten);

              (* Check if ALL data was written to disk.              *)
    if (woBytesWritten <> length(stTemp)) then
      ErrorHandler(7, true);

              (* Advance encoded output file line-counter by 2.      *)
    inc(byLineCount, 2)
  end;        (* MainHeader                                          *)


  (***** PostIt binary to encoded ASCII output-file routine.         *)
  procedure EncodePI34({input }
                          var INbuff         : INbuffType;
                              woINbuffSize   : word;
                     {output}
                          var OUTbuff        : OUTbuffType;
                          var woOUTbuffIndex : word);
  const
    woTemp : word = 0;
  var
    woBytesProcPage : word;
  begin
              (* Reset the number of binary bytes processed variable.*)
    woBytesProcPage := 0;

              (* Repeat... ...Until entire in-buffer page block has  *)
              (* been processed.                                     *)
    repeat
              (* If this is a new line, then add pre-fix to it.      *)
      if ((woOUTbuffIndex - woExtraIndex) mod coLineWidth = 0) then
        begin
          OUTbuff[woOUTbuffIndex + 1] := 'G';
          OUTbuff[woOUTbuffIndex + 2] := '"';
          inc(woOUTbuffIndex, 2);
          inc(woExtraIndex, 2)
        end;

              (* Prepare temporary word for another byte.            *)
      woTemp := woTemp + (INbuff[succ(woINbuffIndex)] shl byCurrentBit);

              (* Advance IN-buffer index by 1,current-bit word by 8  *)
      inc(woBytesProcPage);
      inc(woINbuffIndex);
      inc(byCurrentBit, 8);

              (* While current-bit word is greater than 5, do...     *)
      while (byCurrentBit > 5) do
        begin
              (* Add the next character to the out-buffer.           *)
          OUTbuff[succ(woOUTbuffIndex)] := PostItSet[(woTemp and 63)];

              (* Calculate new checksum.                             *)
          woChecksum := (woChecksum + (woTemp and 63)) shl 1;
          woChecksum := (woChecksum shr 8) + (woCheckSum and 255);

              (* Remove 6 bits from the temporary word variable.     *)
          woTemp := woTemp shr 6;

              (* Reduce current-bit word by 6.                       *)
          dec(byCurrentBit, 6);

              (* Advance out-buffer index by 1.                      *)
          inc(woOUTbuffIndex, 1);
          inc(woCharsOUT, 1)
        end;

              (* If a complete line has been processed, then add     *)
              (* suffix to it.                                       *)
      if ((woOUTbuffIndex - woExtraIndex) mod coLineWidth = 0) then
        begin
          OUTbuff[woOUTbuffIndex + 1] := '"';
          OUTbuff[woOUTbuffIndex + 2] := coCr;
          OUTbuff[woOUTbuffIndex + 3] := coLf;

              (* Advance encoding variables.                         *)
          inc(woOUTbuffIndex, 3);
          inc(woExtraIndex, 3);
          inc(byLineCount)
        end;

              (* Repeat... ...Until entire in-buffer page block has  *)
              (* been processed.                                     *)
    until (woBytesProcPage = woINbuffSize);

              (* If any bits are left...                             *)
    if (byCurrentBit > 0) then
      begin
              (* Add the next character to the out-buffer.           *)
        OUTbuff[succ(woOUTbuffIndex)] := PostItSet[(woTemp and 63)];

              (* Calculate new checksum.                             *)
        woChecksum := (woChecksum + (woTemp and 63)) shl 1;
        woChecksum := (woChecksum shr 8) + (woChecksum and 255);

              (* Advance out-buffer index by 1.                      *)
        inc(woOUTbuffIndex);
        inc(woCharsOUT)
      end;

              (* If the last line was not complete then add suffix.  *)
    if (((woOUTbuffIndex - woExtraIndex) mod coLineWidth) <> 0) then
      begin
        OUTbuff[woOUTbuffIndex + 1] := '"';
        OUTbuff[woOUTbuffIndex + 2] := coCr;
        OUTbuff[woOUTbuffIndex + 3] := coLf;

              (* Advance encoded output buffer variable by 3         *)
        inc(woOUTbuffIndex, 3)
      end
  end;        (* EncodePI34                                          *)


  (***** Write encoded output file footer.                           *)
  procedure WriteFooter;
  begin
    stTemp := 'N=' + Int2Str(woBinSize, 1) + ':K=255:IF LEN(C$)<>' +
              Int2Str(woCharsOUT, 1) + ' THEN ?"' +
              'Incomplete script file!":BEEP:END' + coCrLf +
              'FOR A=1 TO N:LOCATE ,1:?(100&*A)\N"% done";:IF L=0'+
              ' THEN GOSUB G:L=6' + coCrLf +
              'W=T\P(6-L):GOSUB G:W=W OR T*P(L):L=L-2:B$=CHR$(W AND'+
              ' K):PUT 1,,B$:NEXT' + coCrLf;
    blockwrite(fiOUT, stTemp[1], length(stTemp), woBytesWritten);

              (* Check if ALL data was written to disk.              *)
    if (woBytesWritten <> length(stTemp)) then
      ErrorHandler(7, true);

    stTemp := '?:IF C<>' + Int2Str(woCheckSum, 1) + ' THEN ?"Bad' +
              ' checksum!":BEEP:END ELSE ?"Success!":END' + coCrLf +
              'G:I=I+1:T=INSTR(T$,MID$(C$,I,1))-1:C=(C+T)*2:C=C\256' +
              '+(C AND 255):RETURN' + coCrLf +
              'SUB G(A$):SHARED C$:C$=C$+LEFT$(A$,' +
              Int2Str(coLineWidth,1) + '):END SUB' + coCrLf;
    blockwrite(fiOUT, stTemp[1], length(stTemp), woBytesWritten);

              (* Check if ALL data was written to disk.              *)
    if (woBytesWritten <> length(stTemp)) then
      ErrorHandler(7, true);

    close(fiOUT);
    close(fiIN)
  end;        (* WriteFooter.                                        *)


  (***** Encode complete binary file to ASCII output files.          *)
  procedure EncodeFile;
  const
    woBytesProcTotal : word = 0;
  var
              (* Binary input bytes-to-read index.                   *)
    woReadIndex : word;
  begin
              (* Set extra-character index to the value of the       *)
              (* encoded output buffer index.                        *)
    woExtraIndex := woOUTbuffIndex;

              (* Repeat... ...Until complete binary input file has   *)
              (* been encoded.                                       *)
    repeat
              (* Calculate the number of binary input bytes to read. *)
      woReadIndex := (byPageLength - succ(byLineCount)) * 51;

              (* Clear the binary input and encoded output buffers.  *)
      fillchar(INbuffer, succ(woReadIndex), 0);
      fillchar(OUTbuffer, sizeof(OUTbuffer), 0);

              (* Read in the page-block number of binary input bytes.*)
      blockread(fiIN, INbuffer, woReadIndex, woBytesRead);

              (* Advance the binary input bytes-processed variable.  *)
      inc(woBytesProcTotal, woBytesRead);

              (* Encode the binary bytes in the input-buffer.        *)
      EncodePI34(INbuffer, woBytesRead, OUTbuffer, woOUTbuffIndex);

              (* Write the encoded output buffer to the output file. *)
      blockwrite(fiOUT, OUTbuffer, woOUTbuffIndex, woBytesWritten);

              (* Check if ALL data was written to disk.              *)
      if (woBytesWritten <> woOUTbuffIndex) then
        ErrorHandler(7, true);

              (* Write the encoded output file "footer".             *)
      stTemp := '''End of Page ' + Int2Str(byFileNumber, 1) +
                 coCrLf;
      blockwrite(fiOUT, stTemp[1], length(stTemp));

              (* Check if ALL data was written to disk.              *)
      if (woBytesWritten <> woOUTbuffIndex) then
        ErrorHandler(7, true);

              (* Advance the encoded output line-counter variable.   *)
      inc(byLineCount);

              (* If the encoded output line-counter variable is      *)
              (* is equal to the page-length size, then start        *)
              (* another encoded output file.                        *)
      if (byLineCount = byPageLength) then
        begin
          inc(byFileNumber);
          CreateNewOUTfile(byFileNumber)
        end;

              (* Repeat... ...Until complete binary input file has   *)
              (* been encoded.                                       *)
    until (woBytesProcTotal = woBinSize);

              (* Write encoded output file "footer".                 *)
    WriteFooter

  end;        (* EncodeFile                                          *)


              (* Main program execution block.                       *)
BEGIN
  Title;
  Init;
  OpenInFile;
  CreateNewOUTfile(byFileNumber);
  MainHeader;
  EncodeFile
END.

